#include "userlist.h"
#include <utmp.h>

#define LOCATION MON_UPTIME
#define MAJOR_VERSION 0
#define MINOR_VERSION 1
#define PATCH_VERSION 2

#define PLUGIN_CONFIG_KEYWORD "consolewatch_config"

static gchar* PLUGIN_ABOUT_TEXT = 
				"ConsoleWatch %d.%d.%d\n" \
				"GKrellM consolewatch Plugin\n\n" \
				"Copyright (c) 2000-2002 Mark van Eijk\n" \
				"mark@luon.net\n" \
				"http://gkrellm.luon.net/\n\n" \
				"Released under GNU Public Licence";

static char* ut_type_string[] = {
				"UT_UNKNOWN",
				"RUN_LVL",
				"BOOT_TIME",
				"NEW_TIME",
				"OLD_TIME",
				"INIT_PROCESS",
				"LOGIN",
				"USER_PROCESS",
				"DEAD_PROCESS",
				"ACCOUNTING"
				};

// Configuration variables
/*
static gint tty1;
static gint tty2;
static gint tty3;
static gint tty4;
static gint tty5;
static gint tty6;

static GtkWidget* tty1_button;
static GtkWidget* tty2_button;
static GtkWidget* tty3_button;
static GtkWidget* tty4_button;
static GtkWidget* tty5_button;
static GtkWidget* tty6_button;
*/

// Krell variables
static Panel* tty1Panel;
static Panel* tty2Panel;
static Panel* tty3Panel;
static Panel* tty4Panel;
static Panel* tty5Panel;
static Panel* tty6Panel;

static Decal* tty1Text;
static Decal* tty2Text;
static Decal* tty3Text;
static Decal* tty4Text;
static Decal* tty5Text;
static Decal* tty6Text;

static gint count = 0;

static gint panel_expose_event(GtkWidget* widget, GdkEventExpose* event)
{
	if (widget == tty1Panel->drawing_area)
	{
		gdk_draw_pixmap(widget->window,
			widget->style->fg_gc[GTK_WIDGET_STATE (widget)],
			tty1Panel->pixmap,
			event->area.x, event-> area.y,
			event->area.x, event->area.y,
			event->area.width, event->area.height);
	}
	else if (widget == tty2Panel->drawing_area)
	{
		gdk_draw_pixmap(widget->window,
			widget->style->fg_gc[GTK_WIDGET_STATE (widget)],
			tty2Panel->pixmap,
			event->area.x, event-> area.y,
			event->area.x, event->area.y,
			event->area.width, event->area.height);
	}
	else if (widget == tty3Panel->drawing_area)
	{
		gdk_draw_pixmap(widget->window,
			widget->style->fg_gc[GTK_WIDGET_STATE (widget)],
			tty3Panel->pixmap,
			event->area.x, event-> area.y,
			event->area.x, event->area.y,
			event->area.width, event->area.height);
	}
	else if (widget == tty4Panel->drawing_area)
	{
		gdk_draw_pixmap(widget->window,
			widget->style->fg_gc[GTK_WIDGET_STATE (widget)],
			tty4Panel->pixmap,
			event->area.x, event-> area.y,
			event->area.x, event->area.y,
			event->area.width, event->area.height);
	}
	else if (widget == tty5Panel->drawing_area)
	{
		gdk_draw_pixmap(widget->window,
			widget->style->fg_gc[GTK_WIDGET_STATE (widget)],
			tty5Panel->pixmap,
			event->area.x, event-> area.y,
			event->area.x, event->area.y,
			event->area.width, event->area.height);
	}
	else if (widget == tty6Panel->drawing_area)
	{
		gdk_draw_pixmap(widget->window,
			widget->style->fg_gc[GTK_WIDGET_STATE (widget)],
			tty6Panel->pixmap,
			event->area.x, event-> area.y,
			event->area.x, event->area.y,
			event->area.width, event->area.height);
	}

	return FALSE;
}

void update_decal_text(Panel* panel, Decal* decal, char* tty)
{
	struct utmp searchRec;
	struct utmp* userRec;

	setutent();

	strcpy(searchRec.ut_line, tty);
	if ((userRec = getutline(&searchRec)) != NULL)
	{
		if (userRec->ut_type == USER_PROCESS)
		{
			if (userRec->ut_user == NULL)
			{
				strcpy(userRec->ut_user, "CW_ERROR!");
			}
		}
		else
		{
			strcpy(userRec->ut_user, ut_type_string[userRec->ut_type]);
		}
		gkrellm_draw_decal_text(panel, decal, userRec->ut_user, count);
	}
	else
	{
		gkrellm_draw_decal_text(panel, decal, "NO PROCESS!", count);
	}
}

static void update_plugin(void)
{
	if (GK.second_tick)
	{

		count++;
		
		update_decal_text(tty1Panel, tty1Text, "tty1");
		update_decal_text(tty2Panel, tty2Text, "tty2");
		update_decal_text(tty3Panel, tty3Text, "tty3");
		update_decal_text(tty4Panel, tty4Text, "tty4");
		update_decal_text(tty5Panel, tty5Text, "tty5");
		update_decal_text(tty6Panel, tty6Text, "tty6");

		endutent();

		gkrellm_draw_layers(tty1Panel);
		gkrellm_draw_layers(tty2Panel);
		gkrellm_draw_layers(tty3Panel);
		gkrellm_draw_layers(tty4Panel);
		gkrellm_draw_layers(tty5Panel);
		gkrellm_draw_layers(tty6Panel);
	}
}

static void create_plugin(GtkWidget* vbox, gint first_create)
{
	Style* userStyle;
	TextStyle* userTextStyle;
	GdkImlibImage* userBackground;

	if (first_create)
	{
		tty1Panel = gkrellm_panel_new0();
		tty2Panel = gkrellm_panel_new0();
		tty3Panel = gkrellm_panel_new0();
		tty4Panel = gkrellm_panel_new0();
		tty5Panel = gkrellm_panel_new0();
		tty6Panel = gkrellm_panel_new0();
	}
	else
	{
		gkrellm_destroy_krell_list(tty1Panel);
		gkrellm_destroy_krell_list(tty2Panel);
		gkrellm_destroy_krell_list(tty3Panel);
		gkrellm_destroy_krell_list(tty4Panel);
		gkrellm_destroy_krell_list(tty5Panel);
		gkrellm_destroy_krell_list(tty6Panel);

		gkrellm_destroy_decal_list(tty1Panel);
		gkrellm_destroy_decal_list(tty2Panel);
		gkrellm_destroy_decal_list(tty3Panel);
		gkrellm_destroy_decal_list(tty4Panel);
		gkrellm_destroy_decal_list(tty5Panel);
		gkrellm_destroy_decal_list(tty6Panel);
	}

	userStyle = gkrellm_panel_style(DEFAULT_STYLE);
	userTextStyle = gkrellm_panel_textstyle(DEFAULT_STYLE);
	userBackground = gkrellm_bg_panel_image(DEFAULT_STYLE);
	
	tty1Panel->textstyle = userTextStyle;
	tty2Panel->textstyle = userTextStyle;
	tty3Panel->textstyle = userTextStyle;
	tty4Panel->textstyle = userTextStyle;
	tty5Panel->textstyle = userTextStyle;
	tty6Panel->textstyle = userTextStyle;

	tty1Text = gkrellm_create_decal_text(tty1Panel, "undefined", userTextStyle, userStyle, -1, -1, -1);
	tty2Text = gkrellm_create_decal_text(tty2Panel, "undefined", userTextStyle, userStyle, -1, -1, -1);
	tty3Text = gkrellm_create_decal_text(tty3Panel, "undefined", userTextStyle, userStyle, -1, -1, -1);
	tty4Text = gkrellm_create_decal_text(tty4Panel, "undefined", userTextStyle, userStyle, -1, -1, -1);
	tty5Text = gkrellm_create_decal_text(tty5Panel, "undefined", userTextStyle, userStyle, -1, -1, -1);
	tty6Text = gkrellm_create_decal_text(tty6Panel, "undefined", userTextStyle, userStyle, -1, -1, -1);

	gkrellm_configure_panel(tty1Panel, NULL, userStyle);
	gkrellm_configure_panel(tty2Panel, NULL, userStyle);
	gkrellm_configure_panel(tty3Panel, NULL, userStyle);
	gkrellm_configure_panel(tty4Panel, NULL, userStyle);
	gkrellm_configure_panel(tty5Panel, NULL, userStyle);
	gkrellm_configure_panel(tty6Panel, NULL, userStyle);
	
	gkrellm_create_panel(vbox, tty1Panel, userBackground);
	gkrellm_create_panel(vbox, tty2Panel, userBackground);
	gkrellm_create_panel(vbox, tty3Panel, userBackground);
	gkrellm_create_panel(vbox, tty4Panel, userBackground);
	gkrellm_create_panel(vbox, tty5Panel, userBackground);
	gkrellm_create_panel(vbox, tty6Panel, userBackground);
	
	if (first_create)
	{
		gtk_signal_connect(GTK_OBJECT (tty1Panel->drawing_area), "expose_event", (GtkSignalFunc) panel_expose_event, NULL);
		gtk_signal_connect(GTK_OBJECT (tty2Panel->drawing_area), "expose_event", (GtkSignalFunc) panel_expose_event, NULL);
		gtk_signal_connect(GTK_OBJECT (tty3Panel->drawing_area), "expose_event", (GtkSignalFunc) panel_expose_event, NULL);
		gtk_signal_connect(GTK_OBJECT (tty4Panel->drawing_area), "expose_event", (GtkSignalFunc) panel_expose_event, NULL);
		gtk_signal_connect(GTK_OBJECT (tty5Panel->drawing_area), "expose_event", (GtkSignalFunc) panel_expose_event, NULL);
		gtk_signal_connect(GTK_OBJECT (tty6Panel->drawing_area), "expose_event", (GtkSignalFunc) panel_expose_event, NULL);
	}

	tty1Text->x_off = 0;
	tty2Text->x_off = 0;
	tty3Text->x_off = 0;
	tty4Text->x_off = 0;
	tty5Text->x_off = 0;
	tty6Text->x_off = 0;

	gkrellm_draw_decal_text(tty1Panel, tty1Text, "", 0);
	gkrellm_draw_decal_text(tty2Panel, tty2Text, "", 0);
	gkrellm_draw_decal_text(tty3Panel, tty3Text, "", 0);
	gkrellm_draw_decal_text(tty4Panel, tty4Text, "", 0);
	gkrellm_draw_decal_text(tty5Panel, tty5Text, "", 0);
	gkrellm_draw_decal_text(tty6Panel, tty6Text, "", 0);

	gkrellm_monitor_height_adjust(tty1Panel->h);
	gkrellm_monitor_height_adjust(tty2Panel->h);
	gkrellm_monitor_height_adjust(tty3Panel->h);
	gkrellm_monitor_height_adjust(tty4Panel->h);
	gkrellm_monitor_height_adjust(tty5Panel->h);
	gkrellm_monitor_height_adjust(tty6Panel->h);
	
	gkrellm_draw_layers(tty1Panel);
	gkrellm_draw_layers(tty2Panel);
	gkrellm_draw_layers(tty3Panel);
	gkrellm_draw_layers(tty4Panel);
	gkrellm_draw_layers(tty5Panel);
	gkrellm_draw_layers(tty6Panel);
}

static void save_plugin_config(FILE* configFile)
{
}

static void load_plugin_config(gchar* arg)
{
}

static void apply_plugin_config(void)
{
}

static void create_plugin_tab(GtkWidget* tab_vbox)
{
	GtkWidget* tabs;
	GtkWidget* vbox;
	GtkWidget* aboutLabel;

	gchar* aboutText;

	tabs = gtk_notebook_new();
	gtk_notebook_set_tab_pos(GTK_NOTEBOOK(tabs), GTK_POS_TOP);
	gtk_box_pack_start(GTK_BOX(tab_vbox), tabs, TRUE, TRUE, 0);

	// Setup tab
	/*
	vbox = create_tab(tabs, "Setup");

	tty1_button = gtk_check_button_new_with_label("tty1");
	tty2_button = gtk_check_button_new_with_label("tty2");
	tty3_button = gtk_check_button_new_with_label("tty3");
	tty4_button = gtk_check_button_new_with_label("tty4");
	tty5_button = gtk_check_button_new_with_label("tty5");
	tty6_button = gtk_check_button_new_with_label("tty6");

	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(tty1_button), tty1);
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(tty2_button), tty2);
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(tty3_button), tty3);
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(tty4_button), tty4);
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(tty5_button), tty5);
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(tty6_button), tty6);

	gtk_box_pack_start(GTK_BOX(vbox), tty1_button, FALSE, TRUE, 0);
	gtk_box_pack_start(GTK_BOX(vbox), tty2_button, FALSE, TRUE, 0);
	gtk_box_pack_start(GTK_BOX(vbox), tty3_button, FALSE, TRUE, 0);
	gtk_box_pack_start(GTK_BOX(vbox), tty4_button, FALSE, TRUE, 0);
	gtk_box_pack_start(GTK_BOX(vbox), tty5_button, FALSE, TRUE, 0);
	gtk_box_pack_start(GTK_BOX(vbox), tty6_button, FALSE, TRUE, 0);
	*/

	// About tab
	aboutText = g_strdup_printf(PLUGIN_ABOUT_TEXT, MAJOR_VERSION, MINOR_VERSION, PATCH_VERSION);

	vbox = gkrellm_create_tab(tabs, "About");
	aboutLabel = gtk_label_new(aboutText);
	gtk_container_add(GTK_CONTAINER(vbox), aboutLabel);

	g_free(aboutText);
}

static Monitor plugin_mon = 
	{
		"ConsoleWatch",		// Name
		0,			// ID, 0 if plugin
		create_plugin,		// create function
		update_plugin,		// update function
		create_plugin_tab,	// config tab create function
		apply_plugin_config,	// apply the config function

		save_plugin_config,	// save user config
		load_plugin_config,	// load user config
		PLUGIN_CONFIG_KEYWORD,	// config keyword

		NULL,			// undefined 2
		NULL,			// undefined 1
		NULL,			// undefined 0

		MON_UPTIME,		// insert plugin before this monitor
		NULL,			// handle if plugin (automatic)
		NULL			// path if plugin (automatic)
	};

Monitor* init_plugin(void)
{
	return &plugin_mon;
}


